@@ -1,15 +1,24 @@
 /* -*- c-basic-offset: 4 indent-tabs-mode: nil -*-  vi:set ts=8 sts=4 sw=4: */
 
 /*
-    Rubber Band
+    Rubber Band Library
     An audio time-stretching and pitch-shifting library.
-    Copyright 2007-2010 Chris Cannam.
-    
+    Copyright 2007-2012 Particular Programs Ltd.
+
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
     published by the Free Software Foundation; either version 2 of the
     License, or (at your option) any later version.  See the file
     COPYING included with this distribution for more information.
+
+    Alternatively, if you have a valid commercial licence for the
+    Rubber Band Library obtained by agreement with the copyright
+    holders, you may redistribute and/or modify it under the terms
+    described in that licence.
+
+    If you wish to distribute code using the Rubber Band Library
+    under terms other than those of the GNU General Public License,
+    you must obtain a valid commercial licence before doing so.
 */
 
 #include "rubberband/RubberBandStretcher.h"
@@ -41,6 +50,9 @@ using namespace RubberBand;
 
 #ifdef _WIN32
 using RubberBand::gettimeofday;
+#endif
+
+#ifdef __MSVC__
 using RubberBand::usleep;
 #endif
 
@@ -75,13 +87,15 @@ int main(int argc, char **argv)
     double frequencyshift = 1.0;
     int debug = 0;
     bool realtime = false;
-    bool precise = false;
+    bool precise = true;
     int threading = 0;
     bool lamination = true;
     bool longwin = false;
     bool shortwin = false;
+    bool smoothing = false;
     bool hqpitch = false;
     bool formant = false;
+    bool together = false;
     bool crispchanged = false;
     int crispness = -1;
     bool help = false;
@@ -119,16 +133,19 @@ int main(int argc, char **argv)
             { "crispness",     1, 0, 'c' },
             { "debug",         1, 0, 'd' },
             { "realtime",      0, 0, 'R' },
+            { "loose",         0, 0, 'L' },
             { "precise",       0, 0, 'P' },
             { "formant",       0, 0, 'F' },
             { "no-threads",    0, 0, '0' },
             { "no-transients", 0, 0, '1' },
             { "no-lamination", 0, 0, '2' },
+            { "centre-focus",  0, 0, '7' },
             { "window-long",   0, 0, '3' },
             { "window-short",  0, 0, '4' },
             { "bl-transients", 0, 0, '8' },
             { "detector-perc", 0, 0, '5' },
             { "detector-soft", 0, 0, '6' },
+            { "smoothing",     0, 0, '9' },
             { "pitch-hq",      0, 0, '%' },
             { "threads",       0, 0, '@' },
             { "quiet",         0, 0, 'q' },
@@ -137,7 +154,7 @@ int main(int argc, char **argv)
         };
 
         c = getopt_long(argc, argv,
-                        "t:p:d:RPFc:f:T:D:qhVM:",
+                        "t:p:d:RLPFc:f:T:D:qhVM:",
                         longOpts, &optionIndex);
         if (c == -1) break;
 
@@ -151,6 +168,7 @@ int main(int argc, char **argv)
         case 'f': frequencyshift = atof(optarg); haveRatio = true; break;
         case 'd': debug = atoi(optarg); break;
         case 'R': realtime = true; break;
+        case 'L': precise = false; break;
         case 'P': precise = true; break;
 	case 'F': formant = true; break;
         case '0': threading = 1; break;
@@ -161,7 +179,9 @@ int main(int argc, char **argv)
         case '4': shortwin = true; crispchanged = true; break;
         case '5': detector = PercussiveDetector; crispchanged = true; break;
         case '6': detector = SoftDetector; crispchanged = true; break;
+        case '7': together = true; break;
         case '8': transients = BandLimitedTransients; crispchanged = true; break;
+        case '9': smoothing = true; crispchanged = true; break;
         case '%': hqpitch = true; break;
         case 'c': crispness = atoi(optarg); break;
         case 'q': quiet = true; break;
@@ -179,7 +199,7 @@ int main(int argc, char **argv)
         cerr << endl;
 	cerr << "Rubber Band" << endl;
         cerr << "An audio time-stretching and pitch-shifting library and utility program." << endl;
-	cerr << "Copyright 2010 Chris Cannam.  Distributed under the GNU General Public License." << endl;
+	cerr << "Copyright 2007-2012 Particular Programs Ltd." << endl;
         cerr << endl;
 	cerr << "   Usage: " << argv[0] << " [options] <infile.wav> <outfile.wav>" << endl;
         cerr << endl;
@@ -211,8 +231,9 @@ int main(int argc, char **argv)
         cerr << "crispness parameter are intended to provide the best sounding set of options" << endl;
         cerr << "for most situations.  The default is to use none of these options." << endl;
         cerr << endl;
-        cerr << "  -P,    --precise        Aim for minimal time distortion (implied by -R)" << endl;
-        cerr << "  -R,    --realtime       Select realtime mode (implies -P --no-threads)" << endl;
+        cerr << "  -L,    --loose          Relax timing in hope of better transient preservation" << endl;
+        cerr << "  -P,    --precise        Ignored: The opposite of -L, this is default from 1.6" << endl;
+        cerr << "  -R,    --realtime       Select realtime mode (implies --no-threads)" << endl;
         cerr << "         --no-threads     No extra threads regardless of CPU and channel count" << endl;
         cerr << "         --threads        Assume multi-CPU even if only one CPU is identified" << endl;
         cerr << "         --no-transients  Disable phase resynchronisation at transients" << endl;
@@ -220,9 +241,12 @@ int main(int argc, char **argv)
         cerr << "         --no-lamination  Disable phase lamination" << endl;
         cerr << "         --window-long    Use longer processing window (actual size may vary)" << endl;
         cerr << "         --window-short   Use shorter processing window" << endl;
+        cerr << "         --smoothing      Apply window presum and time-domain smoothing" << endl;
         cerr << "         --detector-perc  Use percussive transient detector (as in pre-1.5)" << endl;
         cerr << "         --detector-soft  Use soft transient detector" << endl;
         cerr << "         --pitch-hq       In RT mode, use a slower, higher quality pitch shift" << endl;
+        cerr << "         --centre-focus   Preserve focus of centre material in stereo" << endl;
+        cerr << "                          (at a cost in width and individual channel quality)" << endl;
         cerr << endl;
         cerr << "  -d<N>, --debug <N>      Select debug level (N = 0,1,2,3); default 0, full 3" << endl;
         cerr << "                          (N.B. debug level 3 includes audible ticks in output)" << endl;
@@ -301,7 +325,9 @@ int main(int argc, char **argv)
             while (i < line.length() && line[i] == ' ') ++i;
             size_t target = atoi(line.substr(i).c_str());
             mapping[source] = target;
-            cerr << "adding mapping from " << source << " to " << target << endl;
+            if (debug > 0) {
+                cerr << "adding mapping from " << source << " to " << target << endl;
+            }
             ++lineno;
         }
         ifile.close();
@@ -359,8 +385,10 @@ int main(int argc, char **argv)
     if (!lamination) options |= RubberBandStretcher::OptionPhaseIndependent;
     if (longwin)     options |= RubberBandStretcher::OptionWindowLong;
     if (shortwin)    options |= RubberBandStretcher::OptionWindowShort;
+    if (smoothing)   options |= RubberBandStretcher::OptionSmoothingOn;
     if (formant)     options |= RubberBandStretcher::OptionFormantPreserved;
     if (hqpitch)     options |= RubberBandStretcher::OptionPitchHighQuality;
+    if (together)    options |= RubberBandStretcher::OptionChannelsTogether;
 
     switch (threading) {
     case 0:
@@ -494,6 +522,10 @@ int main(int argc, char **argv)
 
         bool final = (frame + ibs >= sfinfo.frames);
 
+        if (debug > 2) {
+            cerr << "count = " << count << ", ibs = " << ibs << ", frame = " << frame << ", frames = " << sfinfo.frames << ", final = " << final << endl;
+        }
+
         ts.process(ibuf, count, final);
 
         int avail = ts.available();
