@@ -1,15 +1,24 @@
 /* -*- c-basic-offset: 4 indent-tabs-mode: nil -*-  vi:set ts=8 sts=4 sw=4: */
 
 /*
-    Rubber Band
+    Rubber Band Library
     An audio time-stretching and pitch-shifting library.
-    Copyright 2007-2010 Chris Cannam.
-    
+    Copyright 2007-2012 Particular Programs Ltd.
+
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
     published by the Free Software Foundation; either version 2 of the
     License, or (at your option) any later version.  See the file
     COPYING included with this distribution for more information.
+
+    Alternatively, if you have a valid commercial licence for the
+    Rubber Band Library obtained by agreement with the copyright
+    holders, you may redistribute and/or modify it under the terms
+    described in that licence.
+
+    If you wish to distribute code using the Rubber Band Library
+    under terms other than those of the GNU General Public License,
+    you must obtain a valid commercial licence before doing so.
 */
 
 #ifndef _RUBBERBAND_C_API_H_
@@ -19,9 +28,9 @@
 extern "C" {
 #endif
 
-#define RUBBERBAND_VERSION "1.5.0-gpl"
+#define RUBBERBAND_VERSION "1.8.1"
 #define RUBBERBAND_API_MAJOR_VERSION 2
-#define RUBBERBAND_API_MINOR_VERSION 3
+#define RUBBERBAND_API_MINOR_VERSION 5
 
 /**
  * This is a C-linkage interface to the Rubber Band time stretcher.
@@ -63,12 +72,18 @@ enum RubberBandOption {
     RubberBandOptionWindowShort          = 0x00100000,
     RubberBandOptionWindowLong           = 0x00200000,
 
+    RubberBandOptionSmoothingOff         = 0x00000000,
+    RubberBandOptionSmoothingOn          = 0x00800000,
+
     RubberBandOptionFormantShifted       = 0x00000000,
     RubberBandOptionFormantPreserved     = 0x01000000,
 
     RubberBandOptionPitchHighQuality     = 0x00000000,
     RubberBandOptionPitchHighSpeed       = 0x02000000,
-    RubberBandOptionPitchHighConsistency = 0x04000000
+    RubberBandOptionPitchHighConsistency = 0x04000000,
+
+    RubberBandOptionChannelsApart        = 0x00000000,
+    RubberBandOptionChannelsTogether     = 0x10000000,
 };
 
 typedef int RubberBandOptions;
