@@ -1,19 +1,30 @@
 /* -*- c-basic-offset: 4 indent-tabs-mode: nil -*-  vi:set ts=8 sts=4 sw=4: */
 
 /*
-    Rubber Band
+    Rubber Band Library
     An audio time-stretching and pitch-shifting library.
-    Copyright 2007-2010 Chris Cannam.
-    
+    Copyright 2007-2012 Particular Programs Ltd.
+
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
     published by the Free Software Foundation; either version 2 of the
     License, or (at your option) any later version.  See the file
     COPYING included with this distribution for more information.
+
+    Alternatively, if you have a valid commercial licence for the
+    Rubber Band Library obtained by agreement with the copyright
+    holders, you may redistribute and/or modify it under the terms
+    described in that licence.
+
+    If you wish to distribute code using the Rubber Band Library
+    under terms other than those of the GNU General Public License,
+    you must obtain a valid commercial licence before doing so.
 */
 
 #include "StretcherImpl.h"
 
+using namespace std;
+
 namespace RubberBand {
 
 
@@ -111,7 +122,7 @@ RubberBandStretcher::setMaxProcessSize(size_t samples)
 }
 
 void
-RubberBandStretcher::setKeyFrameMap(const std::map<size_t, size_t> &mapping)
+RubberBandStretcher::setKeyFrameMap(const map<size_t, size_t> &mapping)
 {
     m_d->setKeyFrameMap(mapping);
 }
@@ -166,19 +177,19 @@ RubberBandStretcher::getInputIncrement() const
     return m_d->getInputIncrement();
 }
 
-std::vector<int>
+vector<int>
 RubberBandStretcher::getOutputIncrements() const
 {
     return m_d->getOutputIncrements();
 }
 
-std::vector<float>
+vector<float>
 RubberBandStretcher::getPhaseResetCurve() const
 {
     return m_d->getPhaseResetCurve();
 }
 
-std::vector<int>
+vector<int>
 RubberBandStretcher::getExactTimePoints() const
 {
     return m_d->getExactTimePoints();
