@@ -1,15 +1,24 @@
 /* -*- c-basic-offset: 4 indent-tabs-mode: nil -*-  vi:set ts=8 sts=4 sw=4: */
 
 /*
-    Rubber Band
+    Rubber Band Library
     An audio time-stretching and pitch-shifting library.
-    Copyright 2007-2010 Chris Cannam.
-    
+    Copyright 2007-2012 Particular Programs Ltd.
+
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
     published by the Free Software Foundation; either version 2 of the
     License, or (at your option) any later version.  See the file
     COPYING included with this distribution for more information.
+
+    Alternatively, if you have a valid commercial licence for the
+    Rubber Band Library obtained by agreement with the copyright
+    holders, you may redistribute and/or modify it under the terms
+    described in that licence.
+
+    If you wish to distribute code using the Rubber Band Library
+    under terms other than those of the GNU General Public License,
+    you must obtain a valid commercial licence before doing so.
 */
 
 #ifndef _RUBBERBAND_STRETCHERIMPL_H_
@@ -18,10 +27,13 @@
 #include "rubberband/RubberBandStretcher.h"
 
 #include "dsp/Window.h"
+#include "dsp/SincWindow.h"
 #include "dsp/FFT.h"
-#include "dsp/CompoundAudioCurve.h"
+
+#include "audiocurves/CompoundAudioCurve.h"
 
 #include "base/RingBuffer.h"
+#include "base/Scavenger.h"
 #include "system/Thread.h"
 #include "system/sysutils.h"
 
@@ -32,6 +44,12 @@ using namespace RubberBand;
 namespace RubberBand
 {
 
+#ifdef PROCESS_SAMPLE_TYPE
+typedef PROCESS_SAMPLE_TYPE process_t;
+#else
+typedef double process_t;
+#endif
+
 class AudioCurveCalculator;
 class StretchCalculator;
 
@@ -93,8 +111,10 @@ protected:
     size_t m_sampleRate;
     size_t m_channels;
 
-    size_t consumeChannel(size_t channel, const float *input,
-                          size_t samples, bool final);
+    void prepareChannelMS(size_t channel, const float *const *inputs,
+                          size_t offset, size_t samples, float *prepared);
+    size_t consumeChannel(size_t channel, const float *const *inputs,
+                          size_t offset, size_t samples, bool final);
     void processChunks(size_t channel, bool &any, bool &last);
     bool processOneChunk(); // across all channels, for real time use
     bool processChunkForChannel(size_t channel, size_t phaseIncrement,
@@ -107,7 +127,7 @@ protected:
     void analyseChunk(size_t channel);
     void modifyChunk(size_t channel, size_t outputIncrement, bool phaseReset);
     void formantShiftChunk(size_t channel);
-    void synthesiseChunk(size_t channel);
+    void synthesiseChunk(size_t channel, size_t shiftIncrement);
     void writeChunk(size_t channel, size_t shiftIncrement, bool last);
 
     void calculateSizes();
@@ -118,19 +138,47 @@ protected:
     
     size_t roundUp(size_t value); // to next power of two
 
+    template <typename T, typename S>
+    void cutShiftAndFold(T *target, int targetSize,
+                         S *src, // destructive to src
+                         Window<float> *window) {
+        window->cut(src);
+        const int windowSize = window->getSize();
+        const int hs = targetSize / 2;
+        if (windowSize == targetSize) {
+            v_convert(target, src + hs, hs);
+            v_convert(target + hs, src, hs);
+        } else {
+            v_zero(target, targetSize);
+            int j = targetSize - windowSize/2;
+            while (j < 0) j += targetSize;
+            for (int i = 0; i < windowSize; ++i) {
+                target[j] += src[i];
+                if (++j == targetSize) j = 0;
+            }
+        }
+    }
+
     bool resampleBeforeStretching() const;
     
     double m_timeRatio;
     double m_pitchScale;
 
-    size_t m_windowSize;
+    // n.b. either m_fftSize is an integer multiple of m_windowSize,
+    // or vice versa
+    size_t m_fftSize;
+    size_t m_aWindowSize; //!!! or use m_awindow->getSize() throughout?
+    size_t m_sWindowSize; //!!! or use m_swindow->getSize() throughout?
     size_t m_increment;
     size_t m_outbufSize;
 
     size_t m_maxProcessSize;
     size_t m_expectedInputDuration;
-    
+
+#ifndef NO_THREADING    
     bool m_threaded;
+#endif
+
     bool m_realtime;
     Options m_options;
     int m_debugLevel;
@@ -145,9 +193,13 @@ protected:
     ProcessMode m_mode;
 
     std::map<size_t, Window<float> *> m_windows;
-    Window<float> *m_window;
+    std::map<size_t, SincWindow<float> *> m_sincs;
+    Window<float> *m_awindow;
+    SincWindow<float> *m_afilter;
+    Window<float> *m_swindow;
     FFT *m_studyFFT;
 
+#ifndef NO_THREADING
     Condition m_spaceAvailable;
     
     class ProcessThread : public Thread
@@ -168,6 +220,13 @@ protected:
     typedef std::set<ProcessThread *> ThreadSet;
     ThreadSet m_threadSet;
     
+#if defined HAVE_IPP && !defined USE_SPEEX
+    // Exasperatingly, the IPP polyphase resampler does not appear to
+    // be thread-safe as advertised -- a good reason to prefer the
+    // Speex alternative
+    Mutex m_resamplerMutex;
+#endif
+#endif
 
     size_t m_inputDuration;
     CompoundAudioCurve::Type m_detectorType;
@@ -194,7 +253,7 @@ protected:
     float m_freq1;
     float m_freq2;
 
-    size_t m_baseWindowSize;
+    size_t m_baseFftSize;
     float m_rateMultiple;
 
     void writeOutput(RingBuffer<float> &to, float *from,
@@ -202,7 +261,7 @@ protected:
 
     static int m_defaultDebugLevel;
     static const size_t m_defaultIncrement;
-    static const size_t m_defaultWindowSize;
+    static const size_t m_defaultFftSize;
 };
 
 }
