@@ -1,29 +1,41 @@
 /* -*- c-basic-offset: 4 indent-tabs-mode: nil -*-  vi:set ts=8 sts=4 sw=4: */
 
 /*
-    Rubber Band
+    Rubber Band Library
     An audio time-stretching and pitch-shifting library.
-    Copyright 2007-2010 Chris Cannam.
-    
+    Copyright 2007-2012 Particular Programs Ltd.
+
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
     published by the Free Software Foundation; either version 2 of the
     License, or (at your option) any later version.  See the file
     COPYING included with this distribution for more information.
+
+    Alternatively, if you have a valid commercial licence for the
+    Rubber Band Library obtained by agreement with the copyright
+    holders, you may redistribute and/or modify it under the terms
+    described in that licence.
+
+    If you wish to distribute code using the Rubber Band Library
+    under terms other than those of the GNU General Public License,
+    you must obtain a valid commercial licence before doing so.
 */
 
 #ifndef _PROFILER_H_
 #define _PROFILER_H_
 
 //#define NO_TIMING 1
-
 //#define WANT_TIMING 1
 //#define PROFILE_CLOCKS 1
 
+// Define NO_TIMING or NDEBUG to switch off profilers
 #ifdef NDEBUG
-#ifndef WANT_TIMING
 #define NO_TIMING 1
 #endif
+
+// But we always allow WANT_TIMING to switch them back on again
+#ifdef WANT_TIMING
+#undef NO_TIMING
 #endif
 
 #ifndef NO_TIMING
@@ -37,7 +49,10 @@
 #endif
 #endif
 
+#ifndef NO_TIMING
 #include <map>
+#include <string>
+#endif
 
 namespace RubberBand {
 
@@ -53,6 +68,12 @@ public:
 
     static void dump();
 
+    // Unlike the other functions, this is only defined if NO_TIMING
+    // is not set (because it uses std::string which is otherwise
+    // unused here). So, treat this as a tricksy internal function
+    // rather than an API call and guard any call to it appropriately.
+    static std::string getReport();
+
 protected:
     const char* m_c;
 #ifdef PROFILE_CLOCKS
